@@ -23,6 +23,12 @@ void cpp_autograd_set_grad_mode (bool enabled) {
   lantern_autograd_set_grad_mode(enabled);
 }
 
+// [[Rcpp::export]]
+bool cpp_autograd_is_enabled()
+{
+  return lantern_autograd_is_enabled();  
+}
+
 // [[Rcpp::export]]
 Rcpp::XPtr<XPtrTorchTensor> cpp_tensor_grad (Rcpp::XPtr<XPtrTorchTensor> self) {
   return make_xptr<XPtrTorchTensor>(lantern_Tensor_grad(self->get()));
@@ -67,10 +73,13 @@ void event_loop_thread(std::atomic<bool> &event_loop_running)
 }
 
 // [[Rcpp::export]]
-void cpp_torch_method_backward_self_Tensor (Rcpp::XPtr<XPtrTorchTensor> self, Rcpp::XPtr<XPtrTorchTensor> gradient, bool retain_graph, bool create_graph) {
+void cpp_torch_method__backward_self_Tensor_inputs_TensorList (XPtrTorchTensor self, XPtrTorchTensorList inputs, 
+                                                              XPtrTorchTensor gradient, bool retain_graph, 
+                                                              bool create_graph) {
   
-  auto self_ptr = self->get();
-  auto gradient_ptr = gradient->get();
+  auto self_ptr = self.get();
+  auto gradient_ptr = gradient.get();
+  auto inputs_ptr = inputs.get();
   auto retain_graph_val = retain_graph;
   auto create_graph_val = create_graph;
   
@@ -82,8 +91,8 @@ void cpp_torch_method_backward_self_Tensor (Rcpp::XPtr<XPtrTorchTensor> self, Rc
     
     try
     {
-      lantern_Tensor_backward_tensor_tensor_bool_bool(
-        self_ptr, gradient_ptr, 
+      lantern_Tensor__backward_tensor_tensorlist_tensor_bool_bool(
+        self_ptr, inputs_ptr, gradient_ptr, 
         reinterpret_cast<void*>(&retain_graph_val), 
         reinterpret_cast<void*>(&create_graph_val)
       );
