@@ -5,7 +5,10 @@
 // [[Rcpp::export]]
 std::string cpp_device_type_to_string(Rcpp::XPtr<XPtrTorchDevice> device)
 {
-  return std::string(lantern_Device_type(device->get()));
+  auto s = lantern_Device_type(device->get());
+  auto out = std::string(s);
+  lantern_const_char_delete(s);
+  return out;
 }
 
 // [[Rcpp::export]]
@@ -15,10 +18,10 @@ std::int64_t cpp_device_index_to_int(Rcpp::XPtr<XPtrTorchDevice> device)
 }
 
 // [[Rcpp::export]]
-Rcpp::XPtr<XPtrTorchDevice> cpp_torch_device(std::string type, Rcpp::Nullable<std::int64_t> index)
+XPtrTorchDevice cpp_torch_device(std::string type, Rcpp::Nullable<std::int64_t> index)
 {
   int64_t index64 = index.isNull() ? 0 : Rcpp::as<std::int64_t>(index);
   XPtrTorchDevice device = lantern_Device(type.c_str(), index64, !index.isNull());
 
-  return make_xptr<XPtrTorchDevice>(device);
+  return device;
 }
