@@ -29,3 +29,7 @@ type reinterpret_and_clean (void * x)
   deleter(x);
   return o;
 }
+
+XPtrTorchTensor cpp_tensor_undefined ();
+XPtrTorchTensor to_index_tensor (XPtrTorchTensor t);
+XPtrTorchIndexTensorList to_index_tensor_list (XPtrTorchTensorList x);
