@@ -1,16 +1,5 @@
 libs_path <- getwd()
 
-# wait, because of it's built in parallel this script might run before
-# finishing.
-start <- Sys.time()
-timeout <- 15*60
-
-while(length(dir(libs_path, pattern = "torch\\.")) == 0) {
-  Sys.sleep(4)
-  if (as.numeric(Sys.time() - start) > timeout)
-    break 
-}
-
 for (lib in dir(libs_path, pattern = "torch\\.")) {
   file.copy(file.path(libs_path, lib),
             file.path(libs_path, gsub("torch", "torchpkg", lib)),
